#!/usr/bin/perl -w
###############################################################################
##    Copyright (C) 2002 by Eric Gerbier
##    Bug reports to: gerbier@users.sourceforge.net
##    $Id: afick-tk.pl,v 1.35 2004/05/10 11:17:03 gerbier Exp $
##
##    This program is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
################################################################################
# this program just provide a graphical interface to afick
# it just launch afick commands
################################################################################
# rem : can not work in tainted mode : too many errors from Tk modules
################################################################################

use strict;
use Getopt::Long;    # arg analysis

use Tk;              # interface
use Tk::Balloon;     # context help
use Tk::HList;
use Tk::ItemStyle;
use Tk::LabFrame;       # frame with a label
use Tk::Label;
use Tk::ItemStyle;
use Tk::ProgressBar;    # the progress bar
use Tk::ROText;         # to have read-only text
use Tk::Text;
use Tk::Tree;

# global var
my $version = '1.0-0';
my $change_text;        # text widget for change display
my $warning_text;       # text widget for warnings
my $progress_text;      # text widget for progress file name
my $progress_bar;       # progress bar
my $percent_text;       # text percent
my $balloon;            # context help
my $ToggleProgress   = 1;    # checkbox var
my $progress_percent = 0;    # progress bar var

# command arguments
my $configfile = '';
my $help       = '';
my $verbose    = '';
my $Version    = '';
my $Report_full_newdel;
my $Warn_missing_file;
my $Warn_dead_symlinks;
my $Running;
my $Timing;

# labels for Optionmenu widgets (option configuration)
my @verbose_values     = qw(verbose_config verbose noverbose);
my @missing_values     = qw(missing_files_config missing_files nomissing_files);
my @dead_sym_values    = qw(dead_symlinks_config dead_symlinks nodead_symlinks);
my @full_newdel_values = qw(full_newdel_config full_newdel nofull_newdel);
my @running_files_values =
  qw(running_files_config running_files norunning_files);
my @timing_values = qw(timing_config timing notiming);

# default file
# be careful : same code in afick.pl
my $default_conf;
if ( $^O =~ m/linux/ ) {
	$default_conf = '/etc/afick.conf';
}
elsif ( $^O =~ m/^MSWin/ ) {
	$default_conf = 'xp.conf';
}
else {
	$default_conf = 'afick.conf';
}

my %colors = (
	changed  => 'red',
	new      => 'green',    # to be change
	deleted  => 'blue',     # to be change
	dangling => 'yellow',
	normal   => 'black',
	ok       => 'green',
	warning  => 'red',
	comment  => 'blue',
	link     => 'blue'
);

#####################################################################
sub separator($) {
	my $w = shift (@_);
	$w->Label( -text => '     ' )->pack( -side => 'left' );
}
#####################################################################
# used to clear output screen before each command
sub clear_text($) {
	my $widget = shift (@_);
	$widget->delete( '1.0', 'end' );
}

#####################################################################
# print warnings in warning section and console
sub warning ($) {
	my $text = shift (@_);

	# display warnings in color
	$warning_text->insert( 'insert', "$text\n", 'warning' );
	$warning_text->update();
	warn "$text\n";
}
#####################################################################
# display results in windows
sub display($) {
	my $file_df = shift (@_);    # file descriptor

	# save current cursor
	my $cursor = $change_text->cget('-cursor');

	# now a waiting cursor
	$change_text->configure( -cursor => 'watch' );
	clear_text($change_text);
	$change_text->update();
	clear_text($warning_text);
	$warning_text->update();
	clear_text($progress_text);
	$progress_text->update();

	# configure colors
	$change_text->tagConfigure( 'comment', '-foreground', $colors{'comment'} );
	$change_text->tagConfigure( 'change',  '-foreground', $colors{'changed'} );
	$change_text->tagConfigure( 'new',     '-foreground', $colors{'new'} );
	$change_text->tagConfigure( 'deleted', '-foreground', $colors{'deleted'} );
	$change_text->tagConfigure( 'dangling', '-foreground',
		$colors{'dangling'} );

	my $refresh = 0;
	my $nb      = 0;
	my $total   = 1;    # to avoid a divide by zero
	$progress_percent = 0;
	while (<$file_df>) {
		if ( $_ =~ m/^progress total (\d+)/ ) {

			# get number of file in database
			$total = $1;
		}
		elsif ( $_ =~ m/^progress (.*)/ ) {

			# get current scanned file
			$nb++;
			$progress_percent = ( $nb * 100 ) / $total;
			$progress_bar->update();

			my $txt = sprintf( "%02d%% (%d)", $progress_percent, $nb );
			clear_text($percent_text);
			$percent_text->insert( 'end', $txt );
			$percent_text->update();

			my $fic = $1;
			clear_text($progress_text);
			$progress_text->insert( 'end', $fic );
			$progress_text->update();
		}
		elsif ( $_ =~ m/^WARNING: (.*)/ ) {
			warning($1);
		}
		elsif ( $_ =~ m/^#/ ) {
			$change_text->insert( 'insert', $_, 'comment' );
		}

		# todo : complete log parsing and add colors ?
		else {
			$change_text->insert( 'insert', $_ );
		}

		# rafraichissement regulier
		$refresh++;
		if ( $refresh == 10 ) {
			$refresh = 0;
			$change_text->update();
		}
	}    # while
	close($file_df);

	# go to end of both screens
	$change_text->see('end');
	$warning_text->see('end');

	# restore cursor
	$change_text->configure( -cursor => $cursor );
	clear_text($progress_text);
}
#####################################################################
# used to spawn afick commands
sub do_action($) {
	my $arg = shift (@_);

	$arg .= " -c \"$configfile\"" if ($configfile);

	# force values (overload values in config file)
	if ( $verbose eq $verbose_values[1] ) {
		$arg .= ' --verbose';
	}
	elsif ( $verbose eq $verbose_values[2] ) {
		$arg .= ' --noverbose';
	}

	if ( $Warn_missing_file eq $missing_values[1] ) {
		$arg .= ' --missing_files';
	}
	elsif ( $Warn_missing_file eq $missing_values[2] ) {
		$arg .= ' --nomissing_files';
	}

	if ( $Report_full_newdel eq $full_newdel_values[1] ) {
		$arg .= ' --full_newdel';
	}
	elsif ( $Report_full_newdel eq $full_newdel_values[2] ) {
		$arg .= ' --nofull_newdel';
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= ' --dead_symlinks';
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= ' --nodead_symlinks';
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= ' --dead_symlinks';
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= ' --nodead_symlinks';
	}

	if ( $Running eq $running_files_values[1] ) {
		$arg .= ' --running_files';
	}
	elsif ( $Running eq $running_files_values[2] ) {
		$arg .= ' --norunning_files';
	}

	if ( $Timing eq $timing_values[1] ) {
		$arg .= ' --timing';
	}
	elsif ( $Timing eq $timing_values[2] ) {
		$arg .= ' --notiming';
	}

	# progress Checkbutton
	$arg .= ' --progress' if ($ToggleProgress);

	print "arg = $arg \n" if ( $verbose eq $verbose_values[1] );

	if ( !open( ACTION, "afick.pl $arg  2>&1 |" ) ) {
		warning("can not execute afick.pl : $!");
		return;
	}
	else {
		display(*ACTION);
	}
}

#####################################################################
# general texte display
sub display_message($$$) {
	my $main    = shift (@_);
	my $title   = shift (@_);
	my $baratin = shift (@_);

	my $top = $main->Toplevel( -title => $title );
	$top->Button( -text => 'quit', -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => 'left', -expand => 1, -fill => 'both' );

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# display help page
sub do_help($) {
	my $main = shift;

	my $baratin =
	  'this is a graphical interface to afick (another file integrity checker)
to monitor file system changes

menu buttons :
-----------------
File menu
- save  : save output screen to a local file
- load  : display a saved outpout in outpout screen
- history : open history file
- exit : to quit this interface

Action menu
- init : to create the database
- update : compare and update the database
- compare : compare the files with the database
- print : print database content
Analysis menu
- tree-view : display the change in a tree view

Help menu
- help : this page
- about : legal informations
- wizard : how to use afick
- bind keys : summary of all keyboards

the change section :
--------------------
display file changes

the warning section :
---------------------
display afick errors and warnings

the progress section :
----------------------
can be activated/desactivated by the "display progress" checkbutton
it is useful fo follow afick progress (compare on update mode only)
it display the currently scanned file
then a progress bar, from 0 to 100%, with a line each 10%

configuration section :
-----------------------
a first line to work on configuration file :
- configuration file : to enter a configuration file name
- select... : to select a configuration file with a fileselector widget
- view/edit : to view or edit the selected configuration file

then a line of radio-buttons for option, corresponding to command-line options :
- verbose : produce more verbose output (for debugging purpose)
- missing_files : warn about files in config file but missing
- dead_links : warn about dead symlinks
- full_newdel : produce a full report on new/deleted directories
- running_files : warn about files modified during program run
- timing : print timing statistics

each offers 3 choices :
- keep config file choice (*_config)
- force parameter
- force opposite parameter (no*)
';

	display_message( $main, 'help', $baratin );

}
#####################################################################
# display about page
sub do_about($) {
	my $main = shift;

	my $baratin = "afick-tk version $version : a graphical interface to afick\n
url  : http://afick.sourceforge.net
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>
send remarks or bug reports to gerbier\@users.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

";

	display_message( $main, 'about', $baratin );
}
#####################################################################
# display about page
sub do_bind($) {
	my $main = shift;

	my $baratin = '
alt+a  : action menu
alt+a  : about screen
alt+c  : compare action
alt+f  : file menu
alt+h  : help menu
alt+H  : help screen
alt+i  : init action
alt+k  : bind keys screen
alt+l  : load file
alt+n  : analysis menu
alt+p  : print action
alt+s  : save file
alt+t  : tree-view screen
alt+u  : update action
alt+w  ; wizard screen
alt+x  : exit
alt+y  : history
';

	display_message( $main, 'bind keys', $baratin );
}
#####################################################################
# display about page
sub do_wizard($) {
	my $main = shift;

	my $baratin = 'How to use afick ?

First : create a config file according to your needs :
- afick provide 2 config file sample : xp.conf and linux.conf
- read configuration documentation (afick.conf man page or config.html)
- customize your config file with afick-tk or your favorite editor

Second : initiate your database :
- apply "init" button

you are now ready for afick use : compare, update, print ...
';

	display_message( $main, 'wizard', $baratin );
}
#####################################################################
## begin tree section
#####################################################################
sub do_tree_help($) {
	my $main = shift;

	my $baratin = 'afick tree view allow to see a tree of changed files

colors :
- green for new files
- red for file changes
- blue for deleted files
- yellow for dangling files

actions :
- simple click : display details on selected file
- double click : clear color
';

	display_message( $main, 'tree help', $baratin );
}
#####################################################################
sub create_tree_buttons($) {
	my $mw = shift (@_);
	my $frame_text = $mw->LabFrame( -label => 'statistiques' );
	$frame_text->pack( -expand => 0, -fill => 'x' );

	# new files label
	$frame_text->Label( -text => 'new' )->pack( -side => 'left' );
	my $wnew = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => $colors{'new'}
	);
	$wnew->pack( -side => 'left' );
	$balloon->attach( $wnew, -balloonmsg => 'number of new files' );

	# deleted files label
	$frame_text->Label( -text => 'deleted' )->pack( -side => 'left' );
	my $wdel = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => $colors{'deleted'}
	);
	$wdel->pack( -side => 'left' );
	$balloon->attach( $wdel, -balloonmsg => 'number of deleted files' );

	# changed files label
	$frame_text->Label( -text => 'changed' )->pack( -side => 'left' );
	my $wmod = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => $colors{'changed'}
	);
	$wmod->pack( -side => 'left' );
	$balloon->attach( $wmod, -balloonmsg => 'number of changed files' );

	# dangling files label
	$frame_text->Label( -text => 'dangling' )->pack( -side => 'left' );
	my $wdang = $frame_text->ROText(
		-height     => 1,
		-width      => 7,
		-foreground => $colors{'dangling'}
	);
	$wdang->pack( -side => 'left' );
	$balloon->attach( $wdang, -balloonmsg => 'number of dangling links' );

	separator($frame_text);

	# help button
	my $bhelptree = $frame_text->Button(
		-text    => 'help',
		-command => sub { do_tree_help($mw) }
	)->pack( -side => 'left' );
	$balloon->attach( $bhelptree, -balloonmsg => 'display help on tree view' );

	#quit button
	my $bquit =
	  $frame_text->Button( -text => 'quit', -command => [ $mw => 'destroy' ] )
	  ->pack( -side => 'left' );
	$balloon->attach( $bquit, -balloonmsg => 'quit tree view' );

	return ( $wnew, $wdel, $wmod, $wdang );
}
########################################################################
sub add_tree($$$) {
	my ( $w, $fulldir, $style ) = @_;

	my $parent = '/';
	if ( $^O eq 'MSWin32' ) {
		if ( $fulldir =~ s/^([a-z]:)//i ) {
			$parent = $1;
		}
	}

	add_to_tree( $w, $parent, $parent ) unless $w->infoExists($parent);

	my $cur_parent = $parent;
	my @dirs       = ($cur_parent);
	foreach my $name ( split ( /[\/\\]/, $fulldir ) ) {
		next unless length $name;
		push @dirs, $name;
		my $dir = join ( '/', @dirs );
		add_to_tree( $w, $dir, $name, $cur_parent )
		  unless $w->infoExists($dir);
		$cur_parent = $dir;
	}

	# set color
	my $entry = $parent . $fulldir;
	if ( defined $style ) {
		$w->entryconfigure( $entry, -style => $style );
	}
}

########################################################################
sub add_to_tree {
	my ( $w, $dir, $name, $parent ) = @_;

	my $mode = 'close';

	my @args = ( -text => $name, -data => $name );
	if ($parent) {    # Add in alphabetical order.
		foreach my $sib ( $w->infoChildren($parent) ) {
			if ( $sib gt $dir ) {
				push @args, ( -before => $sib );
				last;
			}
		}
	}

	$w->add( $dir, @args );
	$w->setmode( $dir, $mode );
}

#####################################################################
sub tree_parse($$$$$$) {
	my ( $w_maintree, $wnew, $wdel, $wmod, $wdang, $h_change ) = @_;
	my $nbnew = 0;
	my $nbdel = 0;
	my $nbmod = 0;
	my $nbdan = 0;

	my $name;
	my $summary = 1;
	my $logs    = $change_text->get( '1.0', 'end' );

	my @logs = split ( /\n/, $logs );

	foreach (@logs) {
		chomp();

		if ($summary) {

			# summary part
			if (m/^new.*: (.*)/) {
				add_tree( $w_maintree, $1, 'new' );
				$h_change->{$1}{type} = 'new';
				$nbnew++;
			}
			elsif (m/^deleted.*: (.*)/) {
				add_tree( $w_maintree, $1, 'deleted' );
				$h_change->{$1}{type} = 'deleted';
				$nbdel++;
			}
			elsif (m/^changed.*: (.*)/) {
				add_tree( $w_maintree, $1, 'changed' );
				$h_change->{$1}{type} = 'changed';
				$nbmod++;
			}
			elsif (m/^Dangling.*: (.*)/) {
				add_tree( $w_maintree, $1, 'dangling' );
				$h_change->{$1}{type} = 'dangling';
				$nbdan++;
			}
			elsif (m/^# detailed changes/) {

				# summary part end at first blank line
				$summary = 0;
			}
		}
		else {

			# detailed part
			if (m/^\w.*: (.*)/) {
				$name = $1;
			}
			elsif (m/\t(\w+)\s+: (.*)\t(.*)/) {
				my $field = $1;
				my $old   = $2;
				my $new   = $3;
				$h_change->{$name}{$field} = "$old\t$new";
			}
			elsif (m/\t(.*)\s+: (.*)/) {
				my $field = $1;
				my $old   = $2;
				$h_change->{$name}{$field} = "$old";
			}
		}
	}

	$w_maintree->autosetmode();

	# write in text box
	$wnew->insert( 'end',  $nbnew );
	$wdel->insert( 'end',  $nbdel );
	$wmod->insert( 'end',  $nbmod );
	$wdang->insert( 'end', $nbdan );
}
########################################################################
# to display changes info in display panel
sub tree_display_detail($$$) {
	my $d        = shift;
	my $w_detail = shift;
	my $h_change = shift;

	#print "debug : tree_display_detail d=$d\n";
	$d =~ s!//!/!;    # remove first //

	$w_detail->delete('all');
	my $item = $w_detail->addchild('');
	$w_detail->itemCreate( $item, 0, -itemtype => 'text', -text => $d );

	if ( exists $h_change->{$d} ) {
		my $var = $h_change->{$d};
		$w_detail->itemCreate(
			$item, 1,
			-itemtype => 'text',
			-text     => $var->{type}
		);
		foreach my $elem ( keys %$var ) {
			next if ( $elem eq 'type' );
			$item = $w_detail->addchild('');

			$w_detail->itemCreate(
				$item, 0,
				-itemtype => 'text',
				-text     => $elem
			);

			my @tab = split ( /\t/, $var->{$elem} );
			my $col = 1;
			foreach my $t (@tab) {
				$w_detail->itemCreate(
					$item, $col,
					-itemtype => 'text',
					-text     => $t
				);
				$col++;
			}
		}
	}
}
########################################################################
sub tree_remove_color($$) {
	my $d          = shift;
	my $w_maintree = shift;
	$w_maintree->entryconfigure( $d, -style => 'normal' );
}
#####################################################################
sub create_tree_widget($$) {
	my $mw       = shift (@_);
	my $h_change = shift (@_);

	my $w_maintree = $mw->Scrolled(
		'Tree',
		-itemtype   => 'text',
		-separator  => '/',
		-selectmode => 'single',
		-scrollbars => 'osoe',
		-width      => 35,
		-height     => 35
	);
	$balloon->attach( $w_maintree, -balloonmsg => 'tree of changes files' );

	my $w_details = $mw->Scrolled(
		'HList',
		-header  => 1,
		-columns => 4,
		-width   => 100
	);
	$balloon->attach( $w_details,
		-balloonmsg => 'details about changes files' );

	my @header = ( 'filename / field', 'old value', 'new_value' );
	my $nbcol = scalar(@header) - 1;
	for ( 0 .. $nbcol ) {
		$w_details->header( 'create', $_, -text => $header[$_] );
	}

	# single click : display info
	$w_maintree->configure( -browsecmd =>
		  sub { tree_display_detail( $_[0], $w_details, $h_change ) } );

	# double click : remove color
	$w_maintree->configure(
		-command => sub { tree_remove_color( $_[0], $w_maintree ) } );

	$w_maintree->packAdjust( -side => 'left', -fill => 'both', -delay => 1 );
	$w_details->pack( -side => 'right', -fill => 'both', -expand => 1 );

	# creation des style
	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'normal',
		-foreground => 'black',
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'changed',
		-foreground => $colors{'changed'}
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'new',
		-foreground => $colors{'new'}
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'deleted',
		-foreground => $colors{'deleted'}
	);

	$w_maintree->ItemStyle(
		'text',
		-stylename  => 'dangling',
		-foreground => $colors{'dangling'}
	);
	return ( $w_maintree, $w_details );
}
#####################################################################
sub do_tree($) {
	my $main = shift;
	my %h_change;

	my $mw = $main->Toplevel( -title => 'afick tree view' );

	my ( $wnew, $wdel, $wmod, $wdang ) = create_tree_buttons($mw);
	my ( $w_maintree, $w_details ) = create_tree_widget( $mw, \%h_change );

	tree_parse( $w_maintree, $wnew, $wdel, $wmod, $wdang, \%h_change );
}
#####################################################################
##  load section
#####################################################################
# a global var for do_save_log and do_load_log
my $log_types = [
	[ 'Afick Files', 'afick*', ],
	[ 'log files',   '.log' ],
	[ 'text files',  '.txt' ],
	[ 'All Files',   '*', ],
];
#####################################################################
# save change screen to a log file
sub do_save_log($) {
	my $main = shift;

	my @logs = $change_text->get( '1.0', 'end' );
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) = gmtime(time);
	$year += 1900;
	$mon  += 1;
	my $date     = "${year}_${mon}_${mday}_${hour}_${min}";
	my $logname  = 'afick.' . $date . '.log';
	my $filename = $main->getSaveFile(
		-initialfile => "$logname",
		-filetypes   => $log_types
	);

	if ($filename) {
		print "log in $filename\n";
		if ( !open( LOG, '>', $filename ) ) {
			warning("can not write on $filename");
		}
		else {
			print LOG @logs;
			close LOG;
		}
	}
}
#####################################################################
sub load_file($) {
	my $filename = shift (@_);
	clear_text($change_text);
	clear_text($warning_text);

	if ( !open( LOG, '<', $filename ) ) {
		warning("can not read $filename : $!");
	}
	else {
		display(*LOG);
	}
}
#####################################################################
# load a log file into change part
sub do_load_log($) {
	my $main = shift;

	my @logs = $change_text->get( '1.0', 'end' );
	my $filename = $main->getOpenFile( -filetypes => $log_types );
	if ($filename) {
		load_file($filename);
	}
}
#####################################################################
## history section
#####################################################################
# ask afick.pl about config
# avoid duplicate code on parsing config file and options
sub get_config() {

	# first ask for config
	my $arg = '--print_config';
	$arg .= " -c \"$configfile\"" if ($configfile);

	if ( !open( ACTION, "afick.pl $arg  2>&1 |" ) ) {

		# errors messages are set by caller
		return undef;
	}
	else {
		my %config;

		while (<ACTION>) {
			chomp;
			if (s/^DEBUG: //) {
				my ( $dir, $val ) = split (/=/);
				$config{$dir} = $val;
			}
		}    # while
		close(ACTION);

		return %config;
	}
}
#####################################################################
# simple click : show history
sub click_history($$$$) {
	my $w           = shift (@_);
	my $num         = shift (@_);
	my $archive_dir = shift (@_);
	my $label       = shift (@_);

	# get date
	my $item = $w->itemCget( $num, 0, 'text' );
	if ( $item =~ m/^(\d+)\/(\d+)\/(\d+) (\d+):(\d+):(\d+)/ ) {
		my $log = $archive_dir . '/afick.' . $1 . $2 . $3 . $4 . $5 . $6;

		if ( -f $log ) {
			load_file($log);
			$label->configure( -text => "archive file $log loaded" );
		}
		else {
			$label->configure( -text => "archive file $log not found" );
		}
	}
	else {
		$label->configure( -text => "bad date entry in history : $item" );
	}
}
#####################################################################
# double click : delete archive file
sub delete_archive($$$$) {
	my $w           = shift (@_);
	my $num         = shift (@_);
	my $archive_dir = shift (@_);
	my $label       = shift (@_);

	$label->configure( -text => "call delete_archive $num" );

	# get date
	my $item = $w->itemCget( $num, 0, 'text' );
	if ( $item =~ m/^(\d+)\/(\d+)\/(\d+) (\d+):(\d+):(\d+)/ ) {
		my $log = $archive_dir . '/afick.' . $1 . $2 . $3 . $4 . $5 . $6;

		if ( -f $log ) {
			unlink($log);
			print "delete $log\n";
			$label->configure( -text => "archive file $log deleted" );
		}
		else {
			print "$log not found\n";
			$label->configure( -text => "archive file $log not found" );
		}
	}
	else {
		print "bad date $item\n";
		$label->configure( -text => "bad date entry in history : $item" );
	}
}
#####################################################################
sub clean_history($$) {
	my $label  = shift (@_);
	my $config = shift (@_);
	if ( !-f $config->{'history'} ) {
		$label->configure( -text => 'no history file' );
		return;
	}
	else {
		my $archive_dir = $config->{'archive'};

		if ( !-d $archive_dir ) {
			$label->configure( -text => 'no archive directory' );
			return;
		}
		open( HIST, '<', $config->{'history'} );
		my $newhist = $config->{'history'} . '.new';
		open( HISTNEW, '>', $newhist )
		  or $label->configure(
			-text => "can not open new history $newhist : $!" );
		while ( my $ligne = <HIST> ) {
			my $log;
			my $date;

			# decode date to write links to archive files
			if ( $ligne =~ m/^(\d+)\/(\d+)\/(\d+) (\d+):(\d+):(\d+) (.*)/ ) {
				my $log =
				  $archive_dir . '/afick.' . $1 . $2 . $3 . $4 . $5 . $6;

				if ( -f $log ) {
					print HISTNEW $ligne;
				}
				else {

					# no archive file
					$label->configure( -text => "skip $log" );
				}
			}
			else {

				# bad date format
				$label->configure( -text => "bad date format on $ligne" );
			}
		}    # while
		close(HIST);
		close(HISTNEW);

		# move files
		unlink( $config->{'history'} );
		rename( $newhist, $config->{'history'} );
	}
}
#####################################################################
sub do_history_help($) {
	my $main = shift;

	my $baratin = 'afick history view allow to see old runs
colors are :
- red for runs with changes
- green for "clean" runs without any changes

if the date is in blue, the archive file in available and you can :
- simple click : load selected file
- double click : delete selected file
';

	display_message( $main, 'history help', $baratin );
}
#####################################################################
sub do_history($) {
	my $main = shift;

	my $label;
	my %config;

	my $top = $main->Toplevel( -title => 'history' );

	# frame for all buttons
	my $frame_b = $top->Frame()->pack();
	$frame_b->Button(
		-text    => 'clean',
		-command => sub { clean_history( $label, \%config ); $top->destroy() }
	)->pack( -side => 'left' );
	$frame_b->Button(
		-text    => 'help',
		-command => sub { do_history_help($top) }
	)->pack( -side => 'left' );
	$frame_b->Button( -text => 'quit', -command => [ $top => 'destroy' ] )
	  ->pack( -side => 'left' );

	$label = $top->Label( -width => 72 )->pack();

	%config = get_config();

	if ( !%config ) {
		$label->configure( -text => 'can not read configuration' );
	}
	elsif ( !-f $config{'history'} ) {
		$label->configure( -text => 'no history file' );
	}
	else {
		my $archive_dir = $config{'archive'};

		if ( !-d $archive_dir ) {
			$label->configure( -text => 'no archive directory' );
		}

		my $w_history;
		$w_history = $top->Scrolled(
			'HList',
			-header     => 1,
			-columns    => 3,
			-width      => 100,
			-height     => 20,
			-selectmode => 'single',
			-browsecmd  => sub {
				my $num = shift;
				click_history( $w_history, $num, $archive_dir, $label );
			},
			-command => sub {
				my $num = shift;
				delete_archive( $w_history, $num, $archive_dir, $label );
			}
		)->pack( -expand => 1, -fill => 'both' );
		$balloon->attach( $w_history,
			-balloonmsg => 'history show afick run results' );

		my @header = ( 'date', 'summary', 'details' );
		my $nbcol = scalar(@header) - 1;
		for ( 0 .. $nbcol ) {
			$w_history->header( 'create', $_, -text => $header[$_] );
		}

		# creation des style
		my $style_normal = $w_history->ItemStyle(
			'text',
#			-stylename  => 'normal',
			-foreground => $colors{'normal'}
		);

		my $style_ok = $w_history->ItemStyle(
			'text',
#			-stylename  => 'ok',
			-foreground => $colors{'ok'}
		);

		my $style_change = $w_history->ItemStyle(
			'text',
#			-stylename  => 'changed',
			-foreground => $colors{'changed'}
		);

		my $style_link = $w_history->ItemStyle(
			'text',
#			-stylename  => 'link',
			-foreground => $colors{'link'}
		);

		open( HIST, '<', $config{'history'} );
		while ( my $ligne = <HIST> ) {
			chomp($ligne);
			my $e = $w_history->addchild('');

			my $style = $style_ok;
			if ( $ligne =~ m/files scanned, (\d+) changed/ ) {
				my $nb = $1;
				$style = $style_change if ( $nb != 0 );
			}

			# decode date to write links to archive files
			if ( $ligne =~
				m/^(\d+)\/(\d+)\/(\d+) (\d+):(\d+):(\d+) (.*) (\(.*)/ )
			{

				# format AAAA/MM/JJ HH:MM:SS
				my $date = "$1\/$2\/$3 $4:$5:$6";
				my $log  =
				  $archive_dir . '/afick.' . $1 . $2 . $3 . $4 . $5 . $6;
				my $text   = $7;
				my $detail = $8;

				my $style_date = $style_normal;
				$style_date = $style_link if ( -f $log );
				$w_history->itemCreate(
					$e, 0,
					-itemtype => 'text',
					-text     => $date,
					-style    => $style_date
				);
				$w_history->itemCreate(
					$e, 1,
					-itemtype => 'text',
					-text     => $text,
					-style    => $style
				);
				$w_history->itemCreate(
					$e, 2,
					-itemtype => 'text',
					-text     => $detail,
					-style    => $style_normal
				);
			}
			else {
				$w_history->itemCreate(
					$e, 0,
					-itemtype => 'text',
					-text     => $ligne,
					-style    => $style
				);
			}
		}

		close(HIST);
	}
}
#####################################################################
## config section
#####################################################################
# a global var for select_config and save_config
my $conf_types = [ [ 'config files', '.conf' ], [ 'All Files', '*', ], ];
#####################################################################
# to select a config file
sub select_config($$) {
	my $FenetreP = shift;
	my $entree   = shift;

	my $filename = $FenetreP->getOpenFile( -filetypes => $conf_types );
	if ( defined $filename and $filename ne '' ) {
		$entree->delete( 0, 'end' );
		$entree->insert( 0, $filename );
		$entree->xview('end');
	}
}
#####################################################################
# to save a config file
sub save_config($$$) {
	my $main     = shift;
	my $text     = shift;
	my $top      = shift;
	my $initfile = $configfile;
	$initfile =~ s?/?\\?g;
	my $filename = $main->getSaveFile(
		-initialfile => "$initfile",
		-filetypes   => $conf_types
	);

	my @conf = $text->get( '1.0', 'end' );
	if ( !open( CONF, '>', $configfile ) ) {
		warning("can not write in $configfile : $!");
	}
	else {
		print CONF @conf;
		close CONF;
	}
	$top->destroy();
}
#####################################################################
# open selected config file
sub open_config($) {
	my $main = shift;

	$configfile = $configfile || $default_conf;

	my $top = $main->Toplevel( -title => $configfile );
	my $text = $top->Scrolled(
		'Text',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => 'left', -expand => 1, -fill => 'both' );

	my $SaveButton = $top->Button(
		-text    => 'save',
		-command => sub { save_config( $main, $text, $top ) }
	);
	$SaveButton->pack();
	$top->Button( -text => 'quit', -command => [ $top => 'destroy' ] )
	  ->pack( -after => $SaveButton );

	if ( !open( CONF, '<', $configfile ) ) {
		warning("can not read $configfile : $!");
		return;
	}
	else {
		while (<CONF>) {
			$text->insert( 'end', $_ );
		}
		close(CONF);
		$text->see('1.0');
	}

}
#####################################################################
sub usage($) {

	my $version = shift (@_);
	print <<EOHELP
afick-tk ($version) : a graphical interface to afick
Usage: afick-tk [options]
Options:
--config_file|-c config
	name of config file to use
--full_newdel|-f
	report full  information on new and deleted directories
--help|-h
	this help
--missing_files|-m
	warn about files declared in config files which does not exists
--running_files|-r
	warn about files modified during program run
--dead_symlinks|-s
	warn about dead symlinks
--timing|-t
	Print timing statistics
--verbose|-v
	verbose mode (for debugging purpose)
--version|-V
	print program version

#################################################################
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 
#################################################################
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
#################################################################
EOHELP
}
#####################################################################
##                       main
#####################################################################

# options
Getopt::Long::Configure('no_ignore_case');
unless (
	GetOptions(
		'config_file|c=s'  => \$configfile,
		'full_newdel|f!'   => \$Report_full_newdel,
		'help|h'           => \$help,
		'missing_files|m!' => \$Warn_missing_file,
		'running_files|r!' => \$Running,
		'dead_symlinks|s!' => \$Warn_dead_symlinks,
		'timing|t!'        => \$Timing,
		'verbose|v!'       => \$verbose,
		'version|V'        => \$Version
	)
  )
{
	usage($version);

	# can not use warning : no widget this time
	die ('incorrect option !');
}

if ($help) {
	usage($version);
	exit;
}

if ($Version) {
	print "afick-tk  version $version\n";
	exit;
}

# tk page
my $main = MainWindow->new( -title => "afick-gui $version" );

$balloon = $main->Balloon();

my $side = 'top';

# we will have
# frame 1 : action
# frame 2 : outpout
# frame 3 : warning
# frame 4 : progress bar
# frame 5 : configuration
# frame 6 : options

# frame 1 : actions
####################
my $frame1 = $main->LabFrame( -label => 'menus', -labelside => 'acrosstop' );
$frame1->pack( -side => $side, -expand => 0, -fill => 'x' );

#     file menu
my $filemenu = $frame1->Menubutton(
	-text      => 'File',
	-underline => 0,
	-tearoff   => 0
)->pack( -side => 'left' );
$balloon->attach( $filemenu,
	-msg => 'Press and hold this button to see the menu.' );

my $historymenu = $filemenu->command(
	-label     => 'history',
	-underline => 6,
	-command   => sub { do_history($main) }
);
$main->bind( '<Alt-Key-y>' => sub { do_history($main); } );
my $loadmenu = $filemenu->command(
	-label     => 'load',
	-underline => 0,
	-command   => sub { do_load_log($main) }
);
$main->bind( '<Alt-Key-l>' => sub { do_load_log($main); } );
my $savemenu = $filemenu->command(
	-label     => 'save',
	-underline => 0,
	-command   => sub { do_save_log($main) }
);
$main->bind( '<Alt-Key-s>' => sub { do_save_log($main); } );
my $exitmenu = $filemenu->command(
	-label     => 'exit',
	-underline => 1,
	-command   => sub { exit; }
);
$main->bind( '<Alt-Key-x>' => sub { exit; } );

my $fmenu = $filemenu->cget( -menu );
$balloon->attach(
	$fmenu,
	-state => 'both',
	-msg   => [
		'consult history',
		'load previous log from file',
		'save log to file',
		'exit from afick-tk',
	]
);

#       action menu
my $actionmenu = $frame1->Menubutton(
	-text      => 'Action',
	-underline => 0,
	-tearoff   => 0
)->pack( -side => 'left' );
$balloon->attach( $actionmenu,
	-msg => 'Press and hold this button to see the menu.' );

my $initmenu = $actionmenu->command(
	-label     => 'init',
	-underline => 0,
	-command   => sub { do_action('-i') }
);
$main->bind( '<Alt-Key-i>' => sub { do_action('-i') } );
my $updatemenu = $actionmenu->command(
	-label     => 'update',
	-underline => 0,
	-command   => sub { do_action('-u') }
);
$main->bind( '<Alt-Key-u>' => sub { do_action('-u') } );
my $comparemenu = $actionmenu->command(
	-label     => 'compare',
	-underline => 0,
	-command   => sub { do_action('-k') }
);
$main->bind( '<Alt-Key-c>' => sub { do_action('-k') } );
my $printmenu = $actionmenu->command(
	-label     => 'print',
	-underline => 0,
	-command   => sub { do_action('-p') }
);
$main->bind( '<Alt-Key-p>' => sub { do_action('-p') } );

my $amenu = $actionmenu->cget( -menu );
$balloon->attach(
	$amenu,
	-state => 'both',
	-msg   => [
		'create database',
		'update database',
		'compare with database',
		'print database'
	]
);

# analysis menu
my $anamenu = $frame1->Menubutton(
	-text      => 'analysis',
	-underline => 1,
	-tearoff   => 0
)->pack( -side => 'left' );
$balloon->attach( $anamenu,
	-msg => 'Press and hold this button to see the menu.' );

my $treemenu = $anamenu->command(
	-label     => 'tree-view',
	-underline => 0,
	-command   => sub { do_tree($main) }
);
$main->bind( '<Alt-Key-t>' => sub { do_tree($main); } );

my $ymenu = $anamenu->cget( -menu );
$balloon->attach(
	$ymenu,
	-state => 'both',
	-msg   => [ 'display changes in a tree view' ]
);

# help menu
my $helpmenu = $frame1->Menubutton(
	-text      => 'help',
	-underline => 0,
	-tearoff   => 0
)->pack( -side => 'left' );
$balloon->attach( $helpmenu,
	-msg => 'Press and hold this button to see the menu.' );

my $hhelpmenu = $helpmenu->command(
	-label     => 'help',
	-underline => 0,
	-command   => sub { do_help($main) }
);
$main->bind( '<Alt-Key-H>' => sub { do_help($main); } );
my $wizardmenu = $helpmenu->command(
	-label     => 'wizard',
	-underline => 0,
	-command   => sub { do_wizard($main) }
);
$main->bind( '<Alt-Key-w>' => sub { do_wizard($main); } );
my $bindmenu = $helpmenu->command(
	-label     => 'bind keys',
	-underline => 5,
	-command   => sub { do_bind($main) }
);
$main->bind( '<Alt-Key-k>' => sub { do_bind($main); } );
my $aboutmenu = $helpmenu->command(
	-label     => 'about',
	-underline => 1,
	-command   => sub { do_about($main) }
);
$main->bind( '<Alt-Key-b>' => sub { do_about($main); } );

my $hmenu = $helpmenu->cget( -menu );
$balloon->attach(
	$hmenu,
	-state => 'both',
	-msg   => [
		'display help text',
		'display about text (author, licence ...)',
		'small wizard display'
	]
);

# frame 2 : text vue
####################
my $frame2 =
  $main->LabFrame( -label => 'changes section', -labelside => 'acrosstop' );
$frame2->pack( -side => $side, -expand => 1, -fill => 'both' );

$change_text = $frame2->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 20,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => $side, -expand => 1, -fill => 'both' );
$balloon->attach( $change_text, -balloonmsg => 'change window' );

# frame 3 : warning vue
####################
my $frame3 =
  $main->LabFrame( -label => 'warnings section', -labelside => 'acrosstop' );
$frame3->pack( -side => $side, -expand => 0, -fill => 'x' );

$warning_text = $frame3->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 5,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => $side, -expand => 1, -fill => 'both' );
$warning_text->tagConfigure( 'warning', '-foreground', $colors{'warning'} );
$balloon->attach( $warning_text, -balloonmsg => 'warning window' );

# frame 4 : progress
####################
my $frame4 =
  $main->LabFrame( -label => 'progress section', -labelside => 'acrosstop' );
$frame4->pack( -side => $side, -expand => 0, -fill => 'x' );

my $btoggleprogress = $frame4->Checkbutton(
	-text     => 'display progress',
	-variable => \$ToggleProgress
)->pack( -side => $side );
$balloon->attach( $btoggleprogress, -balloonmsg => 'toggle progress bar' );

#$frame4->Label( -text => 'scan') ->pack( -side => 'right' );
$progress_text = $frame4->ROText(
	-height => 1,
	-width  => 128,
	-wrap   => 'word'
)->pack( -side => $side );
$balloon->attach( $progress_text,
	-balloonmsg => 'display current scanned file' );

$percent_text = $frame4->ROText(
	-height => 1,
	-width  => 16,
	-wrap   => 'word'
)->pack( -side => 'right' );
$balloon->attach( $percent_text, -balloonmsg => 'display percent progress' );

$progress_bar = $frame4->ProgressBar(
	-length      => 780,
	-colors      => [ 0, 'green' ],
	-troughcolor => 'grey55',
	-variable    => \$progress_percent
)->pack( -side => $side );
$balloon->attach( $progress_bar, -balloonmsg => 'progress bar' );

# frame 5  : configuration file
####################
my $frame5 = $main->LabFrame(
	-label     => 'configuration section',
	-labelside => 'acrosstop'
);
$frame5->pack( -side => $side, -expand => 0, -fill => 'x' );

$frame5->Label( -text => 'configuration file' )->pack( -side => 'left' );
my $entry =
  $frame5->Entry( -textvariable => \$configfile, -width => 30 )
  ->pack( -side => 'left', -expand => 1 );
$balloon->attach( $entry, -balloonmsg => 'enter a configuration file name' );

my $bselect = $frame5->Button(
	-text    => 'Select...',
	-command => sub { select_config( $main, $entry ) }
)->pack( -side => 'left' );
$balloon->attach( $bselect,
	-balloonmsg => 'select a configuration file from a file browse' );

my $bedit = $frame5->Button(
	-text    => 'View/Edit',
	-command => sub { open_config($main) }
)->pack( -side => 'left' );
$balloon->attach( $bedit, -balloonmsg => 'edit the configuration file' );

# frame 6 : options
###################
my $frame6 = $main->LabFrame( -label => 'options', -labelside => 'acrosstop' );
$frame6->pack( -side => $side, -expand => 0, -fill => 'x' );

my $bverbose = $frame6->Optionmenu(
	-options  => [@verbose_values],
	-variable => \$verbose
)->pack( -side => 'left' );
$balloon->attach( $bverbose, -balloonmsg => 'configure verbose parameter' );

my $bmissing = $frame6->Optionmenu(
	-options  => [@missing_values],
	-variable => \$Warn_missing_file
)->pack( -side => 'left' );
$balloon->attach( $bmissing,
	-balloonmsg => 'configure missing_files parameter' );

my $bsymlink = $frame6->Optionmenu(
	-options  => [@dead_sym_values],
	-variable => \$Warn_dead_symlinks
)->pack( -side => 'left' );
$balloon->attach( $bsymlink,
	-balloonmsg => 'configure dead_symlinks parameter' );

my $bfull = $frame6->Optionmenu(
	-options  => [@full_newdel_values],
	-variable => \$Report_full_newdel
)->pack( -side => 'left' );
$balloon->attach( $bfull, -balloonmsg => 'configure full_newdel parameter' );

my $brunning = $frame6->Optionmenu(
	-options  => [@running_files_values],
	-variable => \$Running
)->pack( -side => 'left' );
$balloon->attach( $brunning,
	-balloonmsg => 'configure running_files parameter' );

my $btiming = $frame6->Optionmenu(
	-options  => [@timing_values],
	-variable => \$Timing
)->pack( -side => 'left' );
$balloon->attach( $btiming, -balloonmsg => 'configure timing parameter' );

MainLoop;
